@@ -35,7 +35,9 @@ from .index import get_transform, SequenceBuffer, QuerySequenceBuffer
 from .hierarchy import (
     get_root_leaf_offsets, 
     get_hog_taxon_levels, 
-    get_hog_member_prots
+    get_hog_member_prots,
+    is_taxon_implied,
+    get_children
 )
 
 # (numba does not like nested methods)
@@ -982,7 +984,7 @@ def compute_overlap(fam_highloc, fam_lowloc, k, query_len):
 @numba.njit
 def _place_queries(
     query_offsets, q2fam_off, q2fam_score, q2fam_overlap, fam_tab, q2hog_bestpath, q2hog_scores, overlap, fst, sst,
-    true_tax_off, hog_tab, hog_taxa_buff):
+    true_tax_off, tax_tab, hog_tab, chog_buff):
     '''
     For each query:
         Skip when low sequence overlap (overlap)
@@ -997,6 +999,9 @@ def _place_queries(
     q2hog_score = np.full(q2fam_off.size, -1, dtype=np.float64)
     q2max_hog_score = np.full(q2fam_off.size, -1, dtype=np.float64)
     
+    if true_tax_off is not None:
+        true_tax_lineage = get_root_leaf_offsets(true_tax_off, tax_tab['ParentOff'])[::-1]
+    
     for i in numba.prange(query_offsets.size):
         q = query_offsets[i]
         
@@ -1027,9 +1032,11 @@ def _place_queries(
             # stop if subfamily implies the true taxon
             if true_tax_off is not None:
                 hog_off = np.int(root_hog_off + j)
-                hog_taxa = get_hog_taxon_levels(hog_off, hog_tab, hog_taxa_buff)
-                if np.argwhere(hog_taxa == true_tax_off).size == 1:
+                if is_taxon_implied(true_tax_lineage, hog_off, hog_tab, chog_buff):
                     break
+                #hog_taxa = get_hog_taxon_levels(hog_off, hog_tab, hog_taxa_buff)
+                #if np.argwhere(hog_taxa == true_tax_off).size == 1:
+                #    break
                         
         # skip if no subfamily-score > subfamily-score threshold or if best subfamily-score < family-score threshold
         if (best_j is None) or (best_s < fst):
@@ -1041,42 +1048,85 @@ def _place_queries(
 
     return q2hog_off, q2hog_score, q2max_hog_score
 
+#@numba.njit
+#def get_closest_taxa_from_ref(q2hog_off, ref_taxoff, tax_tab, hog_tab, hog_taxa_buff):
+#    '''
+#    Based on the predicted HOG root and descendant taxa, we find the closest taxon from reference taxon.
+#     - if root-taxon is one of its descendants, we report the HOG root-taxon (older child) (should be root-HOGs when using such stopping criterion)
+#     - if root-taxon is one of its ancestors, we report the first ancestor that is defined within the HOG (based on real hog2taxa)
+#       (basically the speciation before the duplication or loss delimiting the HOG from the reference taxon)
+#     - otherwise, we simply report the HOG root-taxon (still the closer from the reference taxon since on wrong lineage)
+#     - na if not placed    
+#    '''
+#    q2closest_taxon = np.zeros(q2hog_off.size, dtype=np.uint64)
+#    ref_lineage = get_root_leaf_offsets(ref_taxoff, tax_tab['ParentOff'])[::-1][1:]
+#
+#    for i, hog_off in enumerate(q2hog_off):
+#
+#        # not placed
+#        if hog_off == -1:
+#            q2closest_taxon[i] == -1
+#            continue
+#
+#        # get the HOG taxa and the HOG root-taxon
+#        hog_taxa = get_hog_taxon_levels(hog_off, hog_tab, hog_taxa_buff)
+#        root_tax_off = hog_tab['TaxOff'][hog_off]    
+#
+#        # reference taxon in HOG taxa (at level)
+#        if np.argwhere(hog_taxa == ref_taxoff).size == 1:
+#            q2closest_taxon[i] = ref_taxoff
+#
+#        # root-taxon in taxon lineage (more general)
+#        elif np.argwhere(ref_lineage == root_tax_off).size == 1:
+#
+#            # get the closer ancestral taxon in HOG
+#            j = 0
+#            while np.argwhere(hog_taxa == ref_lineage[j]).size == 0:
+#                j += 1
+#            q2closest_taxon[i] = ref_lineage[j]
+#
+#        # root-taxon either in child taxa (more specific) or in a different clade
+#        else:
+#            q2closest_taxon[i] = hog_tab['TaxOff'][hog_off]
+#    
+#    return q2closest_taxon
 @numba.njit
-def get_closest_taxa_from_ref(q2hog_off, ref_taxoff, tax_tab, hog_tab, hog_taxa_buff):
+def get_closest_taxa_from_ref(q2hog_off, ref_taxoff, tax_tab, hog_tab, chog_buff):
     '''
-    Based on the predicted HOG root and descendant taxa, we find the closest taxon from reference taxon.
-     - if root-taxon is one of its descendants, we report the HOG root-taxon (older child) (should be root-HOGs when using such stopping criterion)
-     - if root-taxon is one of its ancestors, we report the first ancestor that is defined within the HOG (based on real hog2taxa)
-       (basically the speciation before the duplication or loss delimiting the HOG from the reference taxon)
-     - otherwise, we simply report the HOG root-taxon (still the closer from the reference taxon since on wrong lineage)
+    Based on the predicted HOG, we find the closest implied taxon from the reference taxon.
+     - if the reference taxon is implied in the HOG (descendant of root-taxon and ancestor of child-HOG taxa), we report the reference taxon (at level).
+     - if the root-taxon is one of its ancestors, we report the first ancestor that is defined within the HOG child-HOG taxa (more general).
+       (basically the speciation before the duplication delimiting the HOG from the reference taxon)
+     - otherwise, we simply report the HOG root-taxon ('more specific' or 'different lineage').
      - na if not placed    
     '''
     q2closest_taxon = np.zeros(q2hog_off.size, dtype=np.uint64)
-    ref_lineage = get_root_leaf_offsets(ref_taxoff, tax_tab['ParentOff'])[::-1][1:]
+    true_tax_lineage = get_root_leaf_offsets(ref_taxoff, tax_tab['ParentOff'])[::-1]
 
     for i, hog_off in enumerate(q2hog_off):
 
         # not placed
         if hog_off == -1:
             q2closest_taxon[i] == -1
-            continue
-
-        # get the HOG taxa and the HOG root-taxon
-        hog_taxa = get_hog_taxon_levels(hog_off, hog_tab, hog_taxa_buff)
-        root_tax_off = hog_tab['TaxOff'][hog_off]    
-
-        # reference taxon in HOG taxa (at level)
-        if np.argwhere(hog_taxa == ref_taxoff).size == 1:
+            continue 
+        
+        # reference taxon implied in HOG (at level)
+        if is_taxon_implied(true_tax_lineage, hog_off, hog_tab, chog_buff):
             q2closest_taxon[i] = ref_taxoff
 
-        # root-taxon in taxon lineage (more general)
-        elif np.argwhere(ref_lineage == root_tax_off).size == 1:
-
+        # root-taxon in ancestors (more general)
+        elif np.argwhere(true_tax_lineage[1:] == hog_tab['TaxOff'][hog_off]).size == 1:
+            
+            # get the closest taxon from the reference taxon among the HOG children
+            child_hog_taxa = np.unique(hog_tab['TaxOff'][get_children(hog_off, hog_tab, chog_buff)])
+            
             # get the closer ancestral taxon in HOG
             j = 0
-            while np.argwhere(hog_taxa == ref_lineage[j]).size == 0:
+            while np.argwhere(child_hog_taxa == true_tax_lineage[j]).size == 0:
                 j += 1
-            q2closest_taxon[i] = ref_lineage[j]
+            
+            # add 1 because we actually take the parent taxa of child_hog_taxa
+            q2closest_taxon[i] = true_tax_lineage[j + 1]
 
         # root-taxon either in child taxa (more specific) or in a different clade
         else:
@@ -1084,6 +1134,7 @@ def get_closest_taxa_from_ref(q2hog_off, ref_taxoff, tax_tab, hog_tab, hog_taxa_
     
     return q2closest_taxon
 
+
 class MergeSearch(object):
     def __init__(self, ki, nthreads=None, low_mem=False, include_extant_genes=False):
     	assert ki.db.mode == "r", "Database must be opened in read mode."
@@ -1120,6 +1171,10 @@ class MergeSearch(object):
     def hog_tab(self):
         return self.db._hog_tab[:]
 
+    @cached_property
+    def tax_tab(self):
+        return self.db._tax_tab[:]
+
     @cached_property
     def level_arr(self):
         return self.db._level_arr[:]	    
@@ -1239,8 +1294,10 @@ class MergeSearch(object):
         self._queryRankHog_bestpath = queryRankHog_bestpath
         self._queryRankHog_scores = queryRankHog_scores
 
-        # store ids of sbuff
+        # store ids and lengths of sbuff
         self._query_ids = sbuff.ids.flatten()
+        self._query_lengths = sbuff.idx[1:] - sbuff.idx[:-1]
+
 
     def store_results(self, filename):
         compr = tables.Filters(complevel=6, complib="blosc", fletcher32=True)
@@ -1278,7 +1335,7 @@ class MergeSearch(object):
         '''
         return _place_queries(
             query_offsets, self._queryFam_ranked[:, 0], self._queryFam_scores[:, 0], self._queryFam_overlaps[:, 0], self.fam_tab, 
-            self._queryRankHog_bestpath[0], self._queryRankHog_scores[0], overlap, fst, sst, ref_taxoff, self.hog_tab, self.db._hog_taxa_buff[:])
+            self._queryRankHog_bestpath[0], self._queryRankHog_scores[0], overlap, fst, sst, ref_taxoff, self.tax_tab, self.hog_tab, self.db._chog_arr[:])
 
     def output_results(self, overlap, fst, sst, ref_taxon):
 
@@ -1293,17 +1350,29 @@ class MergeSearch(object):
         q2hog_off, q2hog_score, q2max_hog_score = self.place_queries(
             query_offsets, overlap, fst, sst, ref_taxoff)
         
-        c = ['qseqid', 'hogid', 'overlap', 'family-score', 'subfamily-score']
+        c = ['qseqid', 'hogid', 'overlap', 'family-score', 'subfamily-score', 'qseqlen', 'subfamily-medianseqlen']
         r = [[x.decode('ascii') if isinstance(x, bytes) else x for x in self._query_ids],
             map(lambda x: self.hog_tab['OmaID'][x].decode('ascii') if x != -1 else 'na', q2hog_off),
             [x if q2hog_off[i] != -1 else 'na' for i, x in enumerate(self._queryFam_overlaps.flatten())], 
             map(lambda x: x if x != -1 else 'na', q2max_hog_score),
-            map(lambda x: x if x != -1 else 'na', q2hog_score)]
+            map(lambda x: x if x != -1 else 'na', q2hog_score),
+            self._query_lengths,
+            map(lambda x: self.hog_tab['MedianSeqLen'][x] if x != -1 else 'na', q2hog_off)]
+        
+        ## temporary to check PANTHER placement
+        #c = ['qseqid', 'hogid', 'overlap', 'family-score', 'subfamily-score', 'qseqlen', 'subfamily-medianseqlen']
+        #r = [[x.decode('ascii') if isinstance(x, bytes) else x for x in self._query_ids],
+        #    map(lambda x: self.hog_tab['OmaID'][x].decode('ascii') if x != -1 else 'na', q2hog_off),
+        #    [x if q2hog_off[i] != -1 else 'na' for i, x in enumerate(self._queryFam_overlaps.flatten())], 
+        #    map(lambda x: x if x != -1 else 'na', q2max_hog_score),
+        #    map(lambda x: x if x != -1 else 'na', q2hog_score),
+        #    self._query_lengths,
+        #    self._query_lengths]
 
         # compute taxonomic congruences 
         if ref_taxon:
             q2closest_taxon = get_closest_taxa_from_ref(
-                q2hog_off, ref_taxoff, tax_tab, self.hog_tab, self.db._hog_taxa_buff[:])
+                q2hog_off, ref_taxoff, tax_tab, self.hog_tab, self.db._chog_arr[:])
             c.insert(2, 'closetax')
             r.insert(2, map(lambda x: tax_tab['ID'][x].decode('ascii') if x != -1 else 'na', q2closest_taxon)) 
 
